/**
 * Copyright (c) 2013 Jorge Puig de la Bellacasa
 * Licencia GNU
 */
package edu.uoc.recetario.arquitectura.businessLogic;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import edu.uoc.recetario.arquitectura.dao.IDao;
import edu.uoc.recetario.arquitectura.dao.ComentarioDao;
import edu.uoc.recetario.arquitectura.pojo.ComentarioPojo;

/**
 * Clase que implementa la lgica de negocio de la tabla COMENTARIO
 * 
 * @author Jorge Puig de la Bellacasa
 * 
 */
public class ComentarioBusinessLogic extends BusinessLogicAbs<ComentarioPojo> {

	/**
	 * Dao de la tabla COMENTARIO
	 */
	private final ComentarioDao dao = new ComentarioDao();


	/**
	 * Lista todos los comentarios correspondientes a la receta cuya clave primaria
	 * contiene la cadena que se pasa por parmetro
	 * @param _con Connection a utilizar
	 * @param _idReceta cadena con el patrn a buscar 
	 * @return un ArrayList de Pojos con el resultado de la bsqueda
	 * @throws SQLException
	 */
	public Collection<ComentarioPojo> listByReceta(Connection _con, Serializable _idReceta)
			throws SQLException {
		Collection<ComentarioPojo> r = null;
		final String query = "SELECT IdComentario, Texto, IdUsuario, IdReceta FROM COMENTARIO WHERE IdReceta = ? ORDER BY IdComentario ASC";
		Collection<Object[]> myL = this.dao.listBySQL(_con, query,
				new Object[] { _idReceta }); 
		if (myL != null) {
			r = new ArrayList<ComentarioPojo>();
			for (Object[] row : myL) {
				ComentarioPojo p = new ComentarioPojo();
				p.setId((Integer)row[0]);
				p.setTexto(row[1].toString());
				p.setIdUsuario(row[2].toString());
				p.setIdReceta((Integer)row[3]);
				r.add(p);
			}
		}
		return r;
	}
	
	/**
	 *  Lista todos los comentarios del usuario cuya clave primaria se pasa por parmetro
	 * @param _con Connection a utilizar
	 * @param _idUsuario valor de la clave primaria del usuario
	 * @return ArrayList de tipo Pojo con los comentarios
	 * @throws SQLException
	 */
	public Collection<ComentarioPojo> listByUsuario(Connection _con, Serializable _idUsuario)
			throws SQLException {
		Collection<ComentarioPojo> r = null;
		final String query = "SELECT IdComentario, Texto, IdUsuario, IdReceta FROM COMENTARIO WHERE IdUsuario = ?";
		Collection<Object[]> myL = this.dao.listBySQL(_con, query,
				new Object[] { _idUsuario }); 
		if (myL != null) {
			r = new ArrayList<ComentarioPojo>();
			for (Object[] row : myL) {
				ComentarioPojo p = new ComentarioPojo();
				p.setId((Integer)row[0]);
				p.setTexto(row[1].toString());
				p.setIdUsuario(row[2].toString());
				p.setIdReceta((Integer)row[3]);
				r.add(p);
			}
		}
		return r;
	}
	
	
	/**
	 * Borra todos los comentarios pertenecientes a la receta cuyo identificador
	 * se pasa por parmetro
	 * @param _con Connection a utilizar
	 * @param _idReceta identificador de receta
	 * @return true si se ha eliminado algn comentario
	 * @throws SQLException
	 */
	public boolean deleteByReceta(Connection _con, Serializable _idReceta)
			throws SQLException {
		boolean r = false;
		try {
			Collection<ComentarioPojo> listaComentarios = listByReceta(_con, _idReceta);
			if (listaComentarios != null){
				Iterator<ComentarioPojo> iterator = listaComentarios.iterator();
				while (iterator.hasNext()){
					ComentarioPojo next = iterator.next();
					r = this.dao.delete(_con, next.getId());
				}
			}
		}
		catch (SQLException e) {
			throw e;
		}
		return r;
	}
	
	/**
	 * Borra todos los comentarios pertenecientes al usuario cuyo identificador
	 * se pasa por parmetro
	 * @param _con Connection a utilizar
	 * @param _idUsuario identificador de usuario
	 * @return true si se ha eliminado algn comentario
	 * @throws SQLException
	 */
	public boolean deleteByUsuario(Connection _con, Serializable _idUsuario) throws SQLException {
		boolean r = false;
		try {
			Collection<ComentarioPojo> listaComentarios = listByUsuario(_con, _idUsuario);
			if (listaComentarios != null){
				Iterator<ComentarioPojo> iterator = listaComentarios.iterator();
				while (iterator.hasNext()){
					ComentarioPojo next = iterator.next();
					r = this.dao.delete(_con, next.getId());
				}
			}
		}
		catch (SQLException e) {
			throw e;
		}
		return r;
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see edu.uoc.recetario.arquitectura.businessLogic.BusinessLogicAbs#getDao()
	 */
	@Override
	protected IDao<ComentarioPojo> getDao() {
		return this.dao;
	}
}
