/**
 * Copyright (c) 2013 Jorge Puig de la Bellacasa
 * Licencia GNU
 */
package edu.uoc.recetario.arquitectura.businessLogic;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import edu.uoc.recetario.arquitectura.dao.IDao;
import edu.uoc.recetario.arquitectura.dao.RecetaDao;
import edu.uoc.recetario.arquitectura.pojo.RecetaPojo;

/**
 * Clase que implementa la lgica de negocio de la tabla RECETA
 * 
 * @author Jorge Puig de la Bellacasa
 * 
 */
public class RecetaBusinessLogic extends BusinessLogicAbs<RecetaPojo> {

	/**
	 * DAO de la tabla RECETA
	 */
	private final RecetaDao dao = new RecetaDao();


	/**
	 * Devuelve un listado de Pojos con todas las recetas cuyo nombre incluye la cadena
	 * pasada por parmetro.
	 * Las recetas se recuperan sin fotografa en caso de tenerla, para evitar transferencias
	 * innecesariamente masivas de datos
	 * @param _con Connection a utilizar
	 * @param _nombreReceta 
	 * @return ArrayList de Pojos con todas las entradas de la tabla
	 *  cuyo nombre contiene la cadena pasada por parmetro
	 * @throws SQLException
	 */
	public Collection<RecetaPojo> listByNombre(Connection _con, Serializable _nombreReceta)
			throws SQLException {
		Collection<RecetaPojo> r = null;
		final String query = "SELECT IdReceta, NomReceta, NumComensales, IdUsuario, Ingredientes, Preparacion, NomFoto FROM RECETA WHERE LOWER(NomReceta) LIKE LOWER(?)";
		Collection<Object[]> myL = this.dao.listBySQL(_con, query,
				new Object[] { "%"+_nombreReceta+"%" });
		if (myL != null) {
			r = new ArrayList<RecetaPojo>();
			for (Object[] row : myL) {
				RecetaPojo p = new RecetaPojo();
				p.setId((Integer)row[0]);
				p.setNombreReceta(row[1].toString());
				p.setComensales((Integer)row[2]);
				p.setIdUsuario(row[3].toString());
				p.setIngredientes(row[4].toString());
				p.setPreparacion(row[5].toString());
				if (row[6] != null){
					p.setNombreFoto(row[6].toString());
				}
				else {
					p.setNombreFoto(null);
				}
				r.add(p);
			}
		}
		return r;
	}
	
	/**
	 * Devuelve todas las recetas del usuario cuyo identificador es pasado por parmetro
	 * @param _con Connection a utilizar
	 * @param _idUsuario identificador del usuario
	 * @return ArrayList de objetos de tipo Pojo con todas las recetas del usuario
	 * @throws SQLException
	 */
	public Collection<RecetaPojo> listByUsuario(Connection _con, Serializable _idUsuario)
			throws SQLException {
		Collection<RecetaPojo> r = null;
		final String query = "SELECT IdReceta, NomReceta, NumComensales, IdUsuario, Ingredientes, Preparacion FROM RECETA WHERE IdUsuario = ?";
		Collection<Object[]> myL = this.dao.listBySQL(_con, query,
				new Object[] { _idUsuario });
		if (myL != null) {
			r = new ArrayList<RecetaPojo>();
			for (Object[] row : myL) {
				RecetaPojo p = new RecetaPojo();
				p.setId((Integer)row[0]);
				p.setNombreReceta(row[1].toString());
				p.setComensales((Integer)row[2]);
				p.setIdUsuario(row[3].toString());
				p.setIngredientes(row[4].toString());
				p.setPreparacion(row[5].toString());
				r.add(p);
			}
		}
		return r;
	}
	
	/**
	 * Borra todas las recetas pertenecientes al usuario cuyo identificador
	 * se pasa por parmetro
	 * @param _con Connection a utilizar
	 * @param _idUsuario identificador de usuario
	 * @return true si se ha eliminado alguna receta
	 * @throws SQLException
	 */
	public boolean deleteByUsuario(Connection _con, Serializable _idUsuario) throws SQLException {
		boolean r = false;
		try {
			Collection<RecetaPojo> listaRecetas = listByUsuario(_con, _idUsuario);
			if (listaRecetas != null){
				Iterator<RecetaPojo> iterator = listaRecetas.iterator();
				while (iterator.hasNext()){
					RecetaPojo next = iterator.next();
					r = this.dao.delete(_con, next.getId());
				}
			}
		}
		catch (SQLException e) {
			throw e;
		}
		return r;
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see edu.uoc.recetario.arquitectura.businessLogic.BusinessLogicAbs#getDao()
	 */
	@Override
	protected IDao<RecetaPojo> getDao() {
		return this.dao;
	}
}
