/**
 * Copyright (c) 2013 Jorge Puig de la Bellacasa
 * Licencia GNU
 */
package edu.uoc.recetario.arquitectura.dao;

import java.io.Serializable;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import edu.uoc.recetario.arquitectura.pojo.IPojo;

/**
 * Clase abstracta que implementa los mtodos de acceso a datos comunes a todas
 * las relaciones de la base de datos. Los DAO especficos heredan de esta clase
 * 
 * @author Jorge Puig de la Bellacasa
 * @param <T> Pojo sobre el que trabaja este DAO
 * 
 */
public abstract class DaoAbs<T extends IPojo> implements IDao<T> {

	/**
	 * (non-Javadoc)
	 * 
	 * @see edu.uoc.recetario.arquitectura.dao.IDao#save(java.sql.Connection,
	 *      edu.uoc.recetario.arquitectura.pojo.IPojo)
	 */
	@Override
	public Object save(Connection _con, T _p) throws SQLException {
		Object r = null;
		String query = "INSERT INTO " + getTableName() + " ("
				+ getStringOfFields() + ") VALUES (" + getValuesString() + ")";
		PreparedStatement st = null;
		ResultSet rs = null;
		try {
			st = _con.prepareStatement(query, Statement.RETURN_GENERATED_KEYS);
			setStatement(st, _p);
			st.executeUpdate();
			rs = st.getGeneratedKeys();
			if (rs.next()) {
				r = rs.getObject(1);
			}
		} finally {
			if (rs != null) {
				try {
					rs.close();
				} catch (SQLException ignore) {
				}
			}
			if (st != null) {
				try {
					st.close();
				} catch (SQLException ignore) {
				}
			}
		}
		return r;
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see edu.uoc.recetario.arquitectura.dao.IDao#update(java.sql.Connection,
	 *      edu.uoc.recetario.arquitectura.pojo.IPojo)
	 */
	@Override
	public void update(Connection _con, T _p) throws SQLException {
		String query = "UPDATE " + getTableName() + " SET " + getString4Set()
				+ " WHERE " + getIdField() + "=?";
		PreparedStatement st = null;
		try {
			st = _con.prepareStatement(query);
			int pos = setStatement(st, _p);
			st.setObject(pos, _p.getId());
			st.executeUpdate();
		} finally {
			if (st != null) {
				try {
					st.close();
				} catch (SQLException ignore) {
				}
			}
		}
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see edu.uoc.recetario.arquitectura.dao.IDao#delete(java.sql.Connection,
	 *      java.io.Serializable)
	 */
	@Override
	public boolean delete(Connection _con, Serializable _pk)
			throws SQLException {
		boolean r = false;
		String query = "DELETE FROM " + getTableName() + " WHERE "
				+ getIdField() + " = ?";
		PreparedStatement st = null;
		try {
			st = _con.prepareStatement(query);
			st.setObject(1, _pk);
			r = (st.executeUpdate() > 0);
			return r;
		} finally {
			if (st != null) {
				try {
					st.close();
				} catch (SQLException ignore) {
				}
			}
		}
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see edu.uoc.recetario.arquitectura.dao.IDao#load(java.sql.Connection,
	 *      java.io.Serializable)
	 */
	@Override
	public T load(Connection _con, Serializable _pk) throws Exception {
		T r = null;
		String query = "SELECT " + getStringOfFields() + " FROM "
				+ getTableName() + " WHERE " + getIdField() + " = ?";
		PreparedStatement st = null;
		ResultSet rs = null;
		try {
			st = _con.prepareStatement(query);
			st.setObject(1, _pk);
			rs = st.executeQuery();
			while (rs.next()) {
				if (r != null) {
					throw new SQLException(
							"Se ha encontrado ms de un registro para el elemento con id ["
									+ _pk + "]");
				}
				r = createPojo(rs);
			}
			return r;
		} finally {
			if (rs != null) {
				try {
					rs.close();
				} catch (SQLException ignore) {
				}
			}
			if (st != null) {
				try {
					st.close();
				} catch (SQLException ignore) {
				}
			}
		}
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see edu.uoc.recetario.arquitectura.dao.IDao#list(java.sql.Connection)
	 */
	@Override
	public Collection<T> list(Connection _con) throws Exception {
		List<T> r = null;
		String query = "SELECT " + getStringOfFields() + " FROM "
				+ getTableName();
		PreparedStatement st = null;
		ResultSet rs = null;
		try {
			st = _con.prepareStatement(query);
			rs = st.executeQuery();
			r = new ArrayList<T>();
			while (rs.next()) {
				T p = createPojo(rs);
				r.add(p);
			}
			return r;
		} finally {
			if (rs != null) {
				try {
					rs.close();
				} catch (SQLException ignore) {
				}
			}
			if (st != null) {
				try {
					st.close();
				} catch (SQLException ignore) {
				}
			}
		}
	}

	/**
	 * (non-Javadoc)
	 * 
	 * @see edu.uoc.recetario.arquitectura.dao.IDao#listBySQL(java.sql.Connection,
	 *      java.lang.String, java.lang.Object[])
	 */
	@Override
	public Collection<Object[]> listBySQL(Connection _con, String _sqlQuery,
			Object[] _params) throws SQLException {
		Collection<Object[]> r = null;
		String query = _sqlQuery;
		PreparedStatement st = null;
		ResultSet rs = null;
		try {
			st = _con.prepareStatement(query);
			if (_params != null && _params.length > 0) {
				for (int i = 0; i < _params.length; i++) {
					st.setObject(i + 1, _params[i]);
				}
			}
			rs = st.executeQuery();
			int size = 0;
			while (rs.next()) {
				if (r == null) {
					r = new ArrayList<Object[]>();
					size = rs.getMetaData().getColumnCount();
				}
				Object[] row = new Object[size];
				for (int i = 0; i < size; i++) {
					row[i] = rs.getObject(i + 1);
				}
				r.add(row);
			}
			return r;
		} finally {
			if (rs != null) {
				try {
					rs.close();
				} catch (SQLException ignore) {
				}
			}
			if (st != null) {
				try {
					st.close();
				} catch (SQLException ignore) {
				}
			}
		}
	}

	/**
	 * Devuelve un nico objeto de tipo String con todos los identificadores de
	 * atributo de la relacin preparado para su uso en consultas SQL de tipo
	 * SELECT
	 * 
	 * @return String
	 */
	protected String getStringOfFields() {
		StringBuffer r = new StringBuffer();
		for (String s : getFields()) {
			r.append(s).append(",");
		}
		r.deleteCharAt(r.length() - 1);
		return r.toString();
	}

	/**
	 * Devuelve un nico objeto de tipo String con todos los identificadores de
	 * atributo de la relacin preparado para su uso en consultas SQL de tipo
	 * UPDATE
	 * 
	 * @return string para SQL statement de tipo UPDATE que genera los campos
	 *         necesarios para usar despus de VALUES
	 */
	protected String getString4Set() {
		StringBuffer r = new StringBuffer();
		for (String s : getFields()) {
			r.append(s).append("=?,");
		}
		r.deleteCharAt(r.length() - 1);
		return r.toString();
	}

	/**
	 * Devuelve una objeto de tipo String para su uso en consultas SQL
	 * 
	 * @return string para SQL statement de tipo INSERT que genera los campos
	 *         necesarios para usar despus de VALUES
	 */
	protected String getValuesString() {
		StringBuffer r = new StringBuffer();
		for (@SuppressWarnings("unused")
		String s : getFields()) {
			r.append("?,");
		}
		r.deleteCharAt(r.length() - 1);
		return r.toString();
	}

	/**
	 * Devuelve el valor del campo {@link #tableName}
	 * 
	 * @return Valor de tipo String del campo {@link #tableName}
	 */
	protected abstract String getTableName();

	/**
	 * Devuelve el valor del campo {@link #idField}
	 * 
	 * @return Valor de tipo String del campo {@link #idField}
	 */
	protected abstract String getIdField();

	/**
	 * Devuelve el valor del campo {@link #fields}
	 * 
	 * @return Valor de tipo String[] del campo {@link #fields}
	 */
	protected abstract String[] getFields();

	/**
	 * 
	 * Devuelve un objeto Pojo equivalente a partir de un ResultSet
	 * 
	 * @param rs ResultSet obtenido mediante una consulta a la base de datos
	 * @return Pojo representando la fila obtenida en la consulta
	 * @throws SQLException
	 * @throws Exception
	 */
	protected abstract T createPojo(ResultSet rs) throws SQLException,
			Exception;

	/**
	 * Introduce los datos de un objeto pojo en un PreparedStatement para su uso
	 * en consultas SQL
	 * 
	 * @param _st PreparedStatement a poblar
	 * @param _p Objeto Pojo con los datos a utilizar
	 * @return el ndice del siguiente elemento del PreparedStatement pasado por
	 *         parmetro
	 * @throws SQLException
	 */
	protected abstract int setStatement(PreparedStatement _st, T _p)
			throws SQLException;

}
