/**
 * Copyright (c) 2013 Jorge Puig de la Bellacasa
 * Licencia GNU
 */
package edu.uoc.recetario.cu00;

import java.sql.Connection;
import java.sql.SQLException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jasypt.util.password.BasicPasswordEncryptor;

import edu.uoc.recetario.arquitectura.ConexionFactory;
import edu.uoc.recetario.arquitectura.businessLogic.UsuarioBusinessLogic;
import edu.uoc.recetario.arquitectura.pojo.UsuarioPojo;

/**
 * CU00 Autentificacin y acceso<br>
 * Implementa los casos de uso
 * <ul>
 * <li>CU01 Iniciar sesion</li>
 * <li>CU02 Registrar Usuario</li>
 * </ul>
 * 
 * @author Jorge Puig de la Bellacasa
 * 
 */
public class CU00Script {

	/** Lgica de usuario */
	static UsuarioBusinessLogic usuarioBusinessLogic = new UsuarioBusinessLogic();
	/** log */
	static final Log LOG = LogFactory.getLog(CU00Script.class);

	/**
	 * <b>CU01 Iniciar sesion</b><br>
	 * Comprueba que el usuario y contrasea existan en el sistema
	 * 
	 * @param _model Modelo con los datos proporcionados por el usuario
	 * @return <code>true</code> si el usuario y contrasea existen en el
	 *         sistema
	 * @throws SQLException
	 * @throws Exception
	 */
	public static UsuarioPojo autentificar(CU01Model _model)
			throws SQLException, Exception {
		UsuarioPojo r = null;
		Connection con = null;
		BasicPasswordEncryptor passwordEncryptor = new BasicPasswordEncryptor();
		try {
			con = ConexionFactory.getConnection();
			UsuarioPojo usuario = usuarioBusinessLogic.load(con, _model
					.getUsuario().getId());
			if (usuario != null) {
				if (passwordEncryptor.checkPassword(_model.getUsuario()
						.getContrasena(), usuario.getContrasena())) {
					r = usuario;
				}
			}
		} catch (Throwable e) {
			logException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}

		return r;
	}

	/**
	 * <b>CU02 Registrar usuario</b><br>
	 * Registra un nuevo usuario en el sistema. El usuario se crea si no existe
	 * otro usuario con el mismo identificador. El nuevo usuario es de clase
	 * "user".
	 * 
	 * @param _model Modelo con los datos proporcionados por el usuario
	 * @return <code>true</code> si el usuario se ha creado
	 * @throws SQLException
	 * @throws Exception
	 */
	public static UsuarioPojo crearUsuario(CU02Model _model)
			throws SQLException, Exception {
		UsuarioPojo r = null;
		Connection con = null;
		try {
			con = ConexionFactory.getConnection();
			UsuarioPojo usuario = usuarioBusinessLogic.load(con,
					_model.getUsuario());
			BasicPasswordEncryptor passwordEncryptor = new BasicPasswordEncryptor();
			if (usuario == null) {
				UsuarioPojo usuarioPojo = new UsuarioPojo();
				usuarioPojo.setId(_model.getUsuario());
				usuarioPojo.setContrasena(passwordEncryptor
						.encryptPassword(_model.getContrasena()));
				usuarioPojo.setEmail(_model.getEmail());
				usuarioPojo.setTipoUsuario("user");
				usuarioBusinessLogic.save(con, usuarioPojo);
				r = usuarioBusinessLogic.load(con, usuarioPojo.getId());
			}
		} catch (Throwable e) {
			logException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}

		return r;
	}

	/**
	 * Comprueba si un usuario existe en el sistema
	 * 
	 * @param _model modelo a utilizar
	 * @return true si el usuario existe
	 * @throws SQLException
	 * @throws Exception
	 */
	public static boolean userExists(CU02Model _model) throws SQLException,
			Exception {
		boolean r = false;
		Connection con = null;
		UsuarioPojo p = null;
		try {
			con = ConexionFactory.getConnection();
			p = usuarioBusinessLogic.load(con, _model.getUsuario());
			if (p != null) {
				r = true;
			}
		} catch (Throwable e) {
			logException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}

		return r;

	}

	/**
	 * Logea la Exception y la vuelve a lanzar
	 * 
	 * @param _t
	 * @throws SQLException
	 * @throws Exception
	 */
	public static void logException(Throwable _t) throws SQLException,
			Exception {
		if (_t instanceof SQLException) {
			LOG.error("error en el acceso a la base de datos", _t);
			throw (SQLException) _t;
		} else {
			LOG.error("error", _t);
			throw new Exception(_t.getClass().getName() + " - "
					+ _t.getMessage(), _t.getCause());
		}
	}
}
