/**
 * Copyright (c) 2013 Jorge Puig de la Bellacasa
 * Licencia GNU
 */
package edu.uoc.recetario.gr00;

import java.io.File;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Collection;

import org.apache.commons.io.FileUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import edu.uoc.recetario.arquitectura.ConexionFactory;
import edu.uoc.recetario.arquitectura.businessLogic.ComentarioBusinessLogic;
import edu.uoc.recetario.arquitectura.businessLogic.RecetaBusinessLogic;
import edu.uoc.recetario.arquitectura.businessLogic.UsuarioBusinessLogic;
import edu.uoc.recetario.arquitectura.dao.IDao;
import edu.uoc.recetario.arquitectura.pojo.ComentarioPojo;
import edu.uoc.recetario.arquitectura.pojo.RecetaPojo;

/**
 * GR00 Gestin de recetas<br>
 * Implementa los casos de uso
 * <ul>
 * <li>GR01 Buscar receta</li>
 * <li>GR02 Visualizar receta</li>
 * <li>GR03 Aadir comentario</li>
 * <li>GR04 Eliminar comentario</li>
 * <li>GR05 Aadir receta</li>
 * <li>GR06 Gestionar recetas</li>
 * <li>GR07 Eliminar receta</li>
 * <li>GR08 Modificar receta</li>
 * </ul>
 * 
 * @author Jorge Puig de la Bellacasa
 * 
 */
public class GR00Script {

	/** lgica de negocio de las recetas */
	static RecetaBusinessLogic recetaBusinessLogic = new RecetaBusinessLogic();
	/** lgica de negocio de los comentarios */
	static ComentarioBusinessLogic comentarioBusinessLogic = new ComentarioBusinessLogic();
	/** lgica de negocio de los usuarios */
	static UsuarioBusinessLogic usuarioBusinessLogic = new UsuarioBusinessLogic();
	/** log */
	static final Log LOG = LogFactory.getLog(IDao.class);

	/**
	 * Retorna las recetas cuyo nombre contiene la cadena especificada en el
	 * modelo (_model.getReceta().getNombreReceta()).
	 * 
	 * @param _model el modelo a utilizar
	 * @return Una Collection con todas las recetas cuyo nombre contiene
	 *         _model.getReceta().getNombreReceta(), o null si la bsqueda no
	 *         produjo resultados
	 * @throws SQLException
	 * @throws Exception
	 */
	public static Collection<RecetaPojo> buscarRecetas(GR00Model _model)
			throws SQLException, Exception {
		Connection con = null;
		Collection<RecetaPojo> recetas = null;
		try {
			con = ConexionFactory.getConnection();
			recetas = recetaBusinessLogic.listByNombre(con, _model.getReceta()
					.getNombreReceta());
		} catch (Throwable e) {
			logException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}
		return recetas;
	}

	/**
	 * Retorna las recetas cuyo identificador de usuario coincide con el
	 * identificador de usuario del modelo pasado por parmetro
	 * (_model.usuario.id).
	 * 
	 * @param _model el modelo a utilizar
	 * @return una Collection con las recetas encontradas, null si no se han
	 *         encontrado recetas
	 * @throws SQLException
	 * @throws Exception
	 */
	public static Collection<RecetaPojo> buscarRecetasUsuario(GR00Model _model)
			throws SQLException, Exception {
		Connection con = null;
		Collection<RecetaPojo> recetas = null;
		try {
			con = ConexionFactory.getConnection();
			recetas = recetaBusinessLogic.listByUsuario(con, _model
					.getUsuario().getId());
		} catch (Throwable e) {
			logException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}
		return recetas;
	}

	/**
	 * Retorna la receta cuyo id coincide con _model.receta.id
	 * 
	 * @param _model modelo a utilizar
	 * @return la receta, null si no existe
	 * @throws SQLException
	 * @throws Exception
	 */
	public static RecetaPojo loadReceta(GR00Model _model) throws SQLException,
			Exception {
		Connection con = null;
		RecetaPojo receta = null;
		try {
			con = ConexionFactory.getConnection();
			receta = recetaBusinessLogic.load(con, _model.getReceta().getId());
		} catch (Throwable e) {
			logException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}
		return receta;
	}

	/**
	 * retorna una Collection con todos los comentarios cuyo identificador de
	 * receta coincide con _model.receta.id
	 * 
	 * @param _model el modelo a utilizar
	 * @return Collection con los resultados, null si no hubo resultados
	 * @throws SQLException
	 * @throws Exception
	 */
	public static Collection<ComentarioPojo> buscarComentarios(GR00Model _model)
			throws SQLException, Exception {
		Connection con = null;
		Collection<ComentarioPojo> comentarios = null;
		try {
			con = ConexionFactory.getConnection();
			comentarios = comentarioBusinessLogic.listByReceta(con, _model
					.getReceta().getId());
		} catch (Throwable e) {
			logException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}
		return comentarios;
	}

	/**
	 * retorna el comentario cuyo identificador coincide con
	 * _model.comentario.id
	 * 
	 * @param _model el modelo a utilizar
	 * @return el comentario, null si no existe
	 * @throws SQLException
	 * @throws Exception
	 */
	public static ComentarioPojo loadComentario(GR00Model _model)
			throws SQLException, Exception {
		Connection con = null;
		ComentarioPojo comentario = null;
		try {
			con = ConexionFactory.getConnection();
			comentario = comentarioBusinessLogic.load(con, _model
					.getComentario().getId());
		} catch (Throwable e) {
			logException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}
		return comentario;
	}

	/**
	 * Crea un nuevo comentario en la base de datos con ComentarioPojo.idUsuario
	 * = _model.usuarioSesion.id y el resto de atributos = _model.comentario
	 * (excepto ComentarioPojo.id, que ser creado por la BD)
	 * 
	 * @param _model el modelo a utilizar
	 * @return ComentarioPojo creado en la BD, null si no se cre el comentario
	 * @throws SQLException
	 * @throws Exception
	 */
	public static ComentarioPojo nuevoComentario(GR00Model _model)
			throws SQLException, Exception {
		Connection con = null;
		ComentarioPojo comentario = new ComentarioPojo();
		ComentarioPojo nuevoComentario = null;
		try {
			con = ConexionFactory.getConnection();
			comentario.setIdUsuario(_model.getUsuarioSesion().getId());
			comentario.setTexto(_model.getComentario().getTexto());
			comentario.setIdReceta(_model.getComentario().getIdReceta());
			Long idLong = (Long) comentarioBusinessLogic.save(con, comentario);
			Integer id = idLong.intValue();
			nuevoComentario = comentarioBusinessLogic.load(con, id);
		} catch (Throwable e) {
			logException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}
		return nuevoComentario;
	}

	/**
	 * Elimina de la BD el comentario con id = _model.comentario.id. Atencin:
	 * Asegurarse de que el usuario que elimina el comentario tiene permisos
	 * para hacerlo corresponde a la Action que llama a este procedimiento
	 * 
	 * @param _model el modelo a utilizar
	 * @return true si se ha eliminado el comentario
	 * @throws SQLException
	 * @throws Exception
	 */
	public static boolean eliminarComentario(GR00Model _model)
			throws SQLException, Exception {
		Connection con = null;
		boolean r = false;
		try {
			con = ConexionFactory.getConnection();
			r = comentarioBusinessLogic.delete(con, _model.getComentario()
					.getId());
		} catch (Throwable e) {
			logException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}
		return r;

	}

	/**
	 * Crea una receta en el sistema segn los datos proporcionados por el
	 * modelo
	 * 
	 * @param _model el modelo a utilizar
	 * @return RecetaPojo creada en la BD, null si no se ha creado la receta
	 * @throws SQLException
	 * @throws Exception
	 */
	public static RecetaPojo crearReceta(GR00Model _model) throws SQLException,
			Exception {
		Connection con = null;
		RecetaPojo receta = new RecetaPojo();
		RecetaPojo r = null;
		receta = _model.getReceta();
		receta.setIdUsuario(_model.getUsuarioSesion().getId());
		receta.setNombreFoto(_model.getFotografiaFileName());
		try {
			con = ConexionFactory.getConnection();
			Long idLong = (Long) recetaBusinessLogic.save(con, receta);
			Integer id = idLong.intValue();
			r = recetaBusinessLogic.load(con, id);
		} catch (Throwable e) {
			logException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}
		if (receta.getNombreFoto() != null && _model.getFotografia() != null) {
			FileUtils.copyFile(_model.getFotografia(),
					new File(_model.getUploadedImagesPath() + "/" + r.getId()
							+ "/" + r.getNombreFoto()));
		}
		return r;
	}

	/**
	 * Retorna un listado con las recetas pertenecientes al usuario de la sesin
	 * actual
	 * 
	 * @param _model modelo a utilizar
	 * @return Collection con las recetas, null si el usuario no tiene recetas
	 *         en el sistema
	 * @throws SQLException
	 * @throws Exception
	 */
	public static Collection<RecetaPojo> buscarRecetasUsuarioSesion(
			GR00Model _model) throws SQLException, Exception {
		Connection con = null;
		Collection<RecetaPojo> recetas = null;
		try {
			con = ConexionFactory.getConnection();
			recetas = recetaBusinessLogic.listByUsuario(con, _model
					.getUsuarioSesion().getId());
		} catch (Throwable e) {
			logException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}
		return recetas;
	}

	/**
	 * Elimina la receta cuyo identificador coincide con _model.receta.id.
	 * Atencin: Corresponde a la Action que llama a este mtodo comprobar si el
	 * usuario tiene permisos para borrar esta receta
	 * 
	 * @param _model el modelo a utilizar
	 * @return true si se ha eliminado la receta
	 * @throws SQLException
	 * @throws Exception
	 */
	public static boolean eliminarReceta(GR00Model _model) throws SQLException,
			Exception {
		Connection con = null;
		boolean r = false;
		try {
			con = ConexionFactory.getConnection();
			RecetaPojo receta = _model.getReceta();
			if (receta != null) {
				comentarioBusinessLogic.deleteByReceta(con, receta.getId());
				if (receta.getNombreFoto() != null) {
					File foto = new File(_model.getUploadedImagesPath() + "/"
							+ receta.getId() + "/" + receta.getNombreFoto());
					if (foto.exists()) {
						foto.delete();
					}
				}
				r = recetaBusinessLogic.delete(con, _model.getReceta().getId());
			}

		} catch (Throwable e) {
			logException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}
		return r;
	}

	/**
	 * Modifica una receta existente segn los datos proporcionados por el
	 * modelo pasado por parmetro. La fotografa no se modifica si no se
	 * encuentra una nueva fotografa en el modelo (_model.getFotografia()
	 * retorna null). Atencin: Corresponde a la Action que llama a este mtodo
	 * comprobar si el usuario tiene permisos para modificar esta receta
	 * 
	 * @param _model el modelo a utilizar
	 * @return un objeto RecetaPojo que representa la fotografa actualizada,
	 *         null si no se ha modificado la receta
	 * @throws SQLException
	 * @throws Exception
	 */
	public static RecetaPojo modificarReceta(GR00Model _model)
			throws SQLException, Exception {
		Connection con = null;
		RecetaPojo recetaModificada = null;
		try {
			con = ConexionFactory.getConnection();
			if (_model.getReceta() != null
					&& _model.getReceta().getId() != null) {
				recetaModificada = _model.getReceta();
				RecetaPojo recetaOriginal = recetaBusinessLogic.load(con,
						recetaModificada.getId());
				recetaModificada.setId(recetaOriginal.getId());
				if (_model.getFotografiaFileName() != null
						&& _model.getFotografia() != null) {
					File fotoAntigua = new File(_model.getUploadedImagesPath()
							+ "/" + recetaOriginal.getId() + "/"
							+ recetaOriginal.getNombreFoto());
					if (fotoAntigua.exists()) {
						fotoAntigua.delete();
					}
					File foto = new File(_model.getUploadedImagesPath() + "/"
							+ recetaModificada.getId() + "/"
							+ _model.getFotografiaFileName());
					FileUtils.copyFile(_model.getFotografia(), foto);
					recetaModificada.setNombreFoto(_model
							.getFotografiaFileName());
				} else {
					recetaModificada.setNombreFoto(recetaOriginal
							.getNombreFoto());
				}
				recetaBusinessLogic.update(con, recetaModificada);
			}
		} catch (Throwable e) {
			logException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}
		return recetaModificada;
	}

	/**
	 * Elimina la fotografa de la receta del modelo pasado por parmetro si la
	 * fotografa existe. Atencin: corresponde a la Action que llama a este
	 * procedimiento comprobar si el usuario de esta sesin tiene permisos para
	 * realizar esta accin
	 * 
	 * @param _model el modelo a utilizar
	 * @return la receta modificada o null si la receta no existe o no tiene
	 *         foto
	 * @throws SQLException
	 * @throws Exception
	 */
	public static RecetaPojo eliminarFoto(GR00Model _model)
			throws SQLException, Exception {
		RecetaPojo r = null;
		Connection con = null;
		try {
			con = ConexionFactory.getConnection();
			RecetaPojo receta = _model.getReceta();
			if (receta != null && receta.getNombreFoto() != null) {
				File fotoAntigua = new File(_model.getUploadedImagesPath()
						+ "/" + receta.getId() + "/" + receta.getNombreFoto());
				if (fotoAntigua.exists()) {
					fotoAntigua.delete();
					receta.setNombreFoto(null);
					recetaBusinessLogic.update(con, receta);
					r = recetaBusinessLogic.load(con, receta.getId());
				}
			}
		} catch (Throwable e) {
			logException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}
		return r;
	}

	/**
	 * Logea la Exception pasada por parmetro y vuelve a lanzarla
	 * 
	 * @param _t
	 * @throws SQLException
	 * @throws Exception
	 */
	private static void logException(Throwable _t) throws SQLException,
			Exception {
		if (_t instanceof SQLException) {
			LOG.error("error en el acceso a la base de datos", _t);
			throw (SQLException) _t;
		} else {
			LOG.error("error", _t);
			throw new Exception(_t.getClass().getName() + " - "
					+ _t.getMessage(), _t.getCause());
		}
	}
}
