package edu.uoc.recetario.gs00;

import java.sql.Connection;
import java.sql.SQLException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import edu.uoc.recetario.arquitectura.ConexionFactory;
import edu.uoc.recetario.arquitectura.businessLogic.ComentarioBusinessLogic;
import edu.uoc.recetario.arquitectura.businessLogic.RecetaBusinessLogic;
import edu.uoc.recetario.arquitectura.businessLogic.UsuarioBusinessLogic;
import edu.uoc.recetario.arquitectura.pojo.UsuarioPojo;

/**
 * GS00 Gestin del sistema<br>
 * Implementa los casos de uso
 * <ul>
 * <li>GS01 Gestionar usuarios</li>
 * <li>GS02 Eliminar usuario</li>
 * <li>GS03 Promocionar usuario</li>
 * <li>GS04 Gestionar recetas</li>
 * <li>GS05 Eliminar receta</li>
 * </ul>
 * 
 * @author Jorge Puig de la Bellacasa
 * 
 */
public class GS00Script {

	/** lgica de negocio de las recetas */
	static RecetaBusinessLogic recetaBusinessLogic = new RecetaBusinessLogic();
	/** lgica de negocio de los comentarios */
	static ComentarioBusinessLogic comentarioBusinessLogic = new ComentarioBusinessLogic();
	/** lgica de negocio de los usuarios */
	static UsuarioBusinessLogic usuarioBusinessLogic = new UsuarioBusinessLogic();
	/** log del sistema */
	static final Log LOG = LogFactory.getLog(GS00Script.class);

	/**
	 * devuelve el usuarioPojo cuyo identificador coincide con el que se
	 * encuentra en el modelo que se pasa por parmetro
	 * 
	 * @param _model modelo a utilizar
	 * @return el UsuarioPojo, null si no existe
	 * @throws SQLException
	 * @throws Exception
	 */
	public static UsuarioPojo loadUsuario(GS00Model _model)
			throws SQLException, Exception {
		Connection con = null;
		UsuarioPojo r = null;
		try {
			con = ConexionFactory.getConnection();
			r = usuarioBusinessLogic.load(con, _model.getUsuario().getId());
		} catch (Throwable e) {
			procesarException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}
		return r;
	}

	/**
	 * Elimina del sistema el usuario determinado por el modelo, junto con sus
	 * recetas y comentarios. Atencin: corresponde a la clase que llama a este
	 * mtodo determinar si el usuario de la sesin tiene permisos para borrar
	 * otros usuarios
	 * 
	 * @param _model el modelo a utilizar
	 * @return "true" si se ha eliminado el usuario, "false" de otro modo
	 * @throws SQLException
	 * @throws Exception
	 */
	public static boolean eliminarUsuario(GS00Model _model)
			throws SQLException, Exception {
		Connection con = null;
		boolean r = false;
		try {
			con = ConexionFactory.getConnection();
			comentarioBusinessLogic.deleteByUsuario(con, _model.getUsuario()
					.getId());
			recetaBusinessLogic.deleteByUsuario(con, _model.getUsuario()
					.getId());
			r = usuarioBusinessLogic.delete(con, _model.getUsuario().getId());
		} catch (Throwable e) {
			procesarException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}
		return r;
	}

	/**
	 * promociona al usuario determinado por el modelo a administrador
	 * ("admin"). Atencin: corresponde a la clase que llama a este mtodo
	 * comprobar si el usuario de esta sesin tiene permisos para realizar esta
	 * tarea
	 * 
	 * @param _model el modelo a utilizar
	 * @return El usuario modificado, null si no se ha modificado el usuario
	 * @throws SQLException
	 * @throws Exception
	 */
	public static UsuarioPojo promocionarUsuario(GS00Model _model)
			throws SQLException, Exception {
		Connection con = null;
		UsuarioPojo r = null;
		try {
			con = ConexionFactory.getConnection();
			UsuarioPojo usuario = usuarioBusinessLogic.load(con, _model
					.getUsuario().getId());
			if (usuario != null) {
				usuario.setTipoUsuario("admin");
				usuarioBusinessLogic.update(con, usuario);
				r = usuarioBusinessLogic.load(con, usuario.getId());
			}

		} catch (Throwable e) {
			procesarException(e);
		} finally {
			try {
				if (con != null) {
					con.close();
				}
			} catch (Exception ignore) {
				LOG.error("error cerrando conexin a la BD", ignore);
			}
		}
		return r;

	}

	/**
	 * Logea la excepcin y la vuelve a lanzar
	 * 
	 * @param _t la excepcin
	 * @throws SQLException
	 * @throws Exception
	 */
	public static void procesarException(Throwable _t) throws SQLException,
			Exception {
		if (_t instanceof SQLException) {
			LOG.error("error en el acceso a la base de datos", _t);
			throw (SQLException) _t;
		} else {
			LOG.error("error", _t);
			throw new Exception(_t.getClass().getName() + " - "
					+ _t.getMessage(), _t.getCause());
		}
	}

}
